local addon, POI = ...

local default_settings = {
	mmx=UIParent:GetWidth()/2,
	mmy=UIParent:GetHeight()/2,
	cfgx=100+UIParent:GetWidth()/2,
	cfgy=UIParent:GetHeight()/2,
	locked = false, custom = {}, verbose = false
}

local _lastUpdate = 0
local _qstUpdate = false

POI.MAINMAPVISIBLE = false

POI.data = {
	["z10D7E74AB6D7B293"] = {	-- Dendrome
		["Raid Tears"] = {
			data = {
				isrc=addon.identifier, itex="img/poi_RaidTear.png", isiz=32
			},
			locs = {
				{x=3313, z=6174},
				{x=2964, z=4304},
				{x=3755, z=5278},
				{x=4168, z=5822},
				{x=4366, z=5249},
				{x=4067, z=3843},
				{x=2340, z=3997},
				{x=3670, z=6119},
				{x=3821, z=3959},
				{x=2463, z=4383},
				{x=3368, z=5461},
			}
		}
	},
	["z2F1E4708BEC6A608"] = {				-- Ashora
		["Raid Tears"] = {
			data = {
				isrc=addon.identifier, itex="img/poi_RaidTear.png", isiz=32
			},
			locs = {
				{x=3691,z=7061},
				{x=2328,z=6270},
				{x=3171,z=7328},
				{x=3347,z=7676},
				{x=4599,z=6404},
				{x=2671,z=5690},
				{x=3996,z=6653},
				{x=4173,z=6526},
				{x=2596,z=6748},
				{x=2407,z=8360},
				{x=3407,z=7038},
				{x=3419,z=7376},
				{x=3370,z=6814},
				{x=4652,z=7378},
				{x=2183,z=8238},
				{x=2186,z=6565},
			}
		}
	},
	["z2F9C9E1FF91F9293"] = {			-- Steppes
		["Raid Tears"] = {
			data = {
				isrc=addon.identifier, itex="img/poi_RaidTear.png", isiz=32
			},
			locs = {
				{x=16388, z=8032},
				{x=17164, z=7942},
				{x=15158, z=8578},
				{x=16331, z=5631},
				{x=16733, z=8459},
				{x=14952, z=6775},
				{x=17068, z=5753},
				{x=14410, z=8266},
				{x=16913, z=6920},
				{x=17278, z=7184},
				{x=16819, z=5464},
			}
		}
	}
}

-- /script Inspect.Addon.Detail("MapPOI").data.pluginRaidTears()

local function MergeTable(o,n)
	for k,v in pairs(n) do
		if type(v) == "table" then
			if o[k] == nil then
				o[k] = {}
			end
	 	 	if type(o[k]) == 'table' then
	 			MergeTable(o[k], n[k])
	 	 	end
		else
			if o[k] == nil then
				o[k] = v
			end
		end
	end
end

function POI.pluginRaidTears()
	for k,v in pairs(Inspect.Map.Detail(Inspect.Map.List())) do
		if v.title and string.find(v.title, "Raid") then
			local x = math.floor(v.coordX)
			local z = math.floor(v.coordZ)
			
			local found = false
			
			if POI.data[LIBZONECHANGE.currentZoneID] then
				for zk,zd in pairs(POI.data[LIBZONECHANGE.currentZoneID]["Raid Tears"].locs) do
					if zd.x == x and zd.z == z then
						found = true
					end
				end
			end
			
			if found == false and MapPOI_Settings.verbose then
				print("New raid tear @ x="..x..",z="..z)
			end
		end
	end
end

local prv_sh = {}
local prv_cn = 0

function POI.pluginStrongholds()
	if LIBZONECHANGE.currentZoneID == "z10D7E74AB6D7B293" then
		local cur_sh = {}
		local change = false
		local tx = Inspect.Time.Real()
		for k,v in pairs(Inspect.Map.Detail(Inspect.Map.List())) do
			if v.title and string.find(v.title, "Empyreal Stronghold") then
				local img = "img/poi_SH62.png"
				if string.find(v.description, "Level 60") then
					img = "img/poi_SH60.png"
				end
				local ky = v.coordX..","..v.coordZ
				if prv_sh[ky] == nil then
					change = true
					if MapPOI_Settings.verbose then
						print("New EMPYREAL STRONGHOLD")
					end
				end
				cur_sh[v.coordX..","..v.coordZ] = {tx=tx, x=v.coordX, z=v.coordZ, t=img}
			end
		end
		
		for k,v in pairs(prv_sh) do
			if cur_sh[k] == nil then
				change = true
			end
		end
		
		local shData = {}
		
		if change then
			prv_sh = cur_sh
			for k,v in pairs(cur_sh) do
				table.insert(shData, {x=v.x, z=v.z, itex=v.t, isrc=addon.identifier, isiz=32})
			end
			Library.LibMapNotes.AddNodes("POI: Stronghold",shData)
		end
	end
end

UI.Native.MapMain:EventAttach(Event.UI.Native.Loaded, function()
	POI.MAINMAPVISIBLE = not POI.MAINMAPVISIBLE
	POI.pluginQuestObjectives()
	_lastUpdate = 0
end, "Event.UI.Native.Loaded")	

function POI.Event_System_Update_Begin(h)
	if POI.MAINMAPVISIBLE then
		local tx = Inspect.Time.Real()
		if tx - _lastUpdate > 1 then
			POI.pluginStrongholds()
			POI.pluginRaidTears()
			_lastUpdate = tx
		end
		if _qstUpdate then
			POI.pluginQuestObjectives()
			_qstUpdate = false
		end
	end
end

function POI.Command_Slash_Register_POI(h, args)
	local r = {}
	for token in string.gmatch(args, "[^%s]+") do
		table.insert(r, token)
	end
	
	local cx = tonumber(r[1])
	local cz = tonumber(r[2])
	
	if cx and cz then
		local s = r[3]
		for x=4, #r do
			s = s.." "..r[x]
		end
		Library.LibMapNotes.AddNodes("POI: Waypoint",{{x=cx, z=cz, n=s, isiz=32, itex="img/poi_WayPoint.png", isrc=addon.identifier}})
	else
		POI.ShowUsage()
	end
end

function POI.ShowUsage()
	print("Usage:")
	print("/poi XXX ZZZ [note] - sets up a temporary POI (waypoint) at XXX,ZZZ")
end

function POI.Event_libZoneChange_Player(h, zn, zid)

	if POI.data[zid] then
		for pk, pv in pairs(POI.data[zid]) do
			local data = {}
			for lk, lv in pairs(pv.locs) do
				table.insert(data, {x=lv.x, z=lv.z, itex=pv.data.itex, isrc=pv.data.isrc, isiz=pv.data.isiz})
			end
			if #data > 0 then
				Library.LibMapNotes.AddNodes("POI: "..pk,data)
			end
		end
	end

	POI.UI.config:SetTitle("POI: "..LIBZONECHANGE.currentZoneName)
	
-- /poi -x 2150 -z 3300 -c orange -t "In the sea!"

	if MapPOI_Settings.custom[zid] == nil then
		MapPOI_Settings.custom[zid] = {}
	else
		POI.SendPOIData()
	end
end

function POI.Event_Addon_SavedVariables_Load_End(h,a)
	if a == addon.identifier then
		if MapPOI_Settings == nil then MapPOI_Settings = {} end
		MergeTable(MapPOI_Settings, default_settings)
		if MINIMAPDOCKER then
			MINIMAPDOCKER.Register(addon.identifier, POI.UI.mm_button)
			POI.UI.mm_button:SetTexture(addon.identifier, "img/mm_poi.png")
		else
			POI.UI.mm_button:SetPoint("TOPLEFT", UIParent, "TOPLEFT", MapPOI_Settings.mmx, MapPOI_Settings.mmy)
			if MapPOI_Settings.locked == false then
				POI.UI.mm_button:SetTexture(addon.identifier, "img/mm_poi_bw.png")
			else
				POI.UI.mm_button:SetTexture(addon.identifier, "img/mm_poi.png")
			end
		end
		POI.UI.config:SetPoint("TOPLEFT", UIParent, "TOPLEFT", MapPOI_Settings.cfgx, MapPOI_Settings.cfgy)
		LibVersionCheck.register(addon.toc.Identifier, addon.toc.Version)
	end
end

function POI.SendPOIData()
	local data = {}
	for k,v in pairs(MapPOI_Settings.custom[LIBZONECHANGE.currentZoneID]) do
		table.insert(data, {x=v.x, z=v.z, isrc=addon.identifier, itex=v.itex, isiz=24, n=v.n})
	end
	if #data > 0 then
		Library.LibMapNotes.AddNodes("Custom POI",data)
	end
end

function POI.AddUserPOI(d)
	local found = false
	for k,v in pairs(MapPOI_Settings.custom[LIBZONECHANGE.currentZoneID]) do
		if v.x == d.x and v.z == d.z then
			found = k
			break
		end
	end
	
	if found == false then
		table.insert(MapPOI_Settings.custom[LIBZONECHANGE.currentZoneID], {x=d.x, z=d.z, itex=d.itex, n=d.n, isrc=addon.identifier})
		print(string.format("Adding new POI @ %d,%d", d.x,d.z))
	else
		MapPOI_Settings.custom[LIBZONECHANGE.currentZoneID][found].itex = d.itex
		MapPOI_Settings.custom[LIBZONECHANGE.currentZoneID][found].n = d.n
		print(string.format("Updating POI @ %d,%d", d.x,d.z))
	end
	POI.ShowCfgPOI()
	POI.SendPOIData()
end

POI.SRCH_OS = 0

function POI.ShowCfgPOI()

	if POI.UI.config:GetVisible() == false then
		return
	end

	POI.UI.tPOIedCx:SetText("")
	POI.UI.tPOIedCz:SetText("")
	POI.UI.tPOIedNote:SetText("")
	POI.UI.iPOIed:SetTexture(addon.identifier, "img/poi_red.png")
	POI.EDIT_ICON_SEL = "img/poi_red.png"
	POI.UI.colsframe:SetVisible(false)
	
	local td = MapPOI_Settings.custom[LIBZONECHANGE.currentZoneID]
	POI.POI_SORTED = {}
	
	local pd = Inspect.Unit.Detail("player")
	
	if pd then	
		local px = pd.coordX
		local pz = pd.coordZ
		for k,v in pairs(td) do
			local dx = px-v.x
			local dz = pz-v.z
			MapPOI_Settings.custom[LIBZONECHANGE.currentZoneID][k].k = k
			table.insert(POI.POI_SORTED, {k=k, x=v.x, z=v.z, n=v.n, itex=v.itex, d=((dx*dx)+(dz*dz))})
		end
		table.sort(POI.POI_SORTED, function(a,b) return a.d < b.d end)
	else
		POI.POI_SORTED = td
	end
	
	local ni = #POI.POI_SORTED
	local sk
	local ek
	
	if ni <= 16 then
		sk = 1
		ek = ni
		POI.UI.sb_bg:SetVisible(false)
	else
		if POI.SRCH_OS + 16 > ni then
			POI.SRCH_OS = ni-16
		end
		
		sk = POI.SRCH_OS + 1
		ek = POI.SRCH_OS + 16

		local bh = POI.UI.sb_bg:GetHeight()
		local sh = (16/ni) * bh
		local soh = bh-sh
		local so = POI.SRCH_OS/(ni-16) * soh

		POI.UI.sb:SetHeight(sh)
		POI.UI.sb:SetPoint("TOPLEFT", POI.UI.sb_bg, "TOPLEFT", 0,so)
		POI.UI.sb_bg:SetVisible(true)
	end
	-- table.insert(data, {x=v.x, z=v.z, isrc=aPOI.POI_SORTEDon.identifier, itex=v.itex, isiz=24, n=v.n})
	for x=sk, ek do
		local k = x - POI.SRCH_OS
		POI.UI.notes[k].f:SetVisible(true)
		POI.UI.notes[k].c:SetText(string.format("%d,%d", POI.POI_SORTED[x].x, POI.POI_SORTED[x].z))
		if POI.POI_SORTED[x].itex then
			POI.UI.notes[k].i:SetTexture(addon.identifier, POI.POI_SORTED[x].itex)
		else
			POI.UI.notes[k].i:SetTexture(addon.identifier, "img/poi_red.png")
		end
		if POI.POI_SORTED[x].n then
			POI.UI.notes[k].t:SetText(POI.POI_SORTED[x].n)
		else
			POI.UI.notes[k].t:SetText("")
		end
	end
	
	for x=ek+1, 16 do
		POI.UI.notes[x].f:SetVisible(false)
	end
end

local function is_mousein(self, h)
	self:SetBackgroundColor(0,0.75,0)
end

local function is_mouseout(self, h)
	self:SetBackgroundColor(0,0,0)
end

local function is_wheelforward(self, h)
	if POI.SRCH_OS > 0 then
		POI.SRCH_OS = POI.SRCH_OS-1
		POI.ShowCfgPOI()
	end
end

local function is_wheelback(self, h)
	if POI.SRCH_OS+16 < #MapPOI_Settings.custom[LIBZONECHANGE.currentZoneID] then
		POI.SRCH_OS = POI.SRCH_OS+1
		POI.ShowCfgPOI()
	end
end

function POI.BuildUI()
	POI.UI = {}
	POI.context = UI.CreateContext(addon.identifier)
	POI.UI.mm_button = UI.CreateFrame("Texture", "POI.UI.mm_button", POI.context)
	POI.UI.mm_button:SetWidth(36)
	POI.UI.mm_button:SetHeight(36)

	if MINIMAPDOCKER == nil then
		POI.UI.mm_button:EventAttach(Event.UI.Input.Mouse.Left.Down, function(self, h)
			if MapPOI_Settings.locked == false then
				local mouseData = Inspect.Mouse()
				self.sx = mouseData.x - POI.UI.mm_button:GetLeft()
				self.sy = mouseData.y - POI.UI.mm_button:GetTop()
				self.MouseDown = true
			end
		end, "Event.UI.Input.Mouse.Left.Down")

		POI.UI.mm_button:EventAttach(Event.UI.Input.Mouse.Left.Up, function(self, h)
			if MapPOI_Settings.locked == false then
				local mouseData = Inspect.Mouse()
				MapPOI_Settings.mmx = POI.UI.mm_button:GetLeft()
				MapPOI_Settings.mmy = POI.UI.mm_button:GetTop()
			end
			self.MouseDown = false
		end, "Event.UI.Input.Mouse.Left.Up")

		POI.UI.mm_button:EventAttach(Event.UI.Input.Mouse.Cursor.Move, function(self, h)
			if MapPOI_Settings.locked == false and self.MouseDown then
				local nx, ny
				local mouseData = Inspect.Mouse()
				nx = mouseData.x - self.sx
				ny = mouseData.y - self.sy
				POI.UI.mm_button:SetPoint("TOPLEFT", UIParent, "TOPLEFT", nx,ny)
			end
		end, "Event.UI.Input.Mouse.Cursor.Move")

		POI.UI.mm_button:EventAttach(Event.UI.Input.Mouse.Right.Click, function(self, h)
			self.MouseDown = false
			MapPOI_Settings.locked = not MapPOI_Settings.locked
			if MapPOI_Settings.locked == false then
				POI.UI.mm_button:SetTexture(addon.identifier, "img/mm_poi_bw.png")
			else
				POI.UI.mm_button:SetTexture(addon.identifier, "img/mm_poi.png")
			end
		end, "Event.UI.Input.Mouse.Right.Click")
	end
	
	POI.UI.mm_button:EventAttach(Event.UI.Input.Mouse.Left.Click, function(self, h)
		if MapPOI_Settings.locked or MINIMAPDOCKER then
			POI.UI.config:SetVisible(not POI.UI.config:GetVisible())
			if POI.UI.config:GetVisible() then
				POI.ShowCfgPOI()
			end
		end
	end, "Event.UI.Input.Mouse.Left.Click")
	
	POI.UI.config = UI.CreateFrame("RiftWindow", "POI.UI.config", POI.context)
	POI.UI.config:SetVisible(false)
	
	local border = POI.UI.config:GetBorder()
	
	border:EventAttach(Event.UI.Input.Mouse.Left.Down, function(self, h)
		local mouseData = Inspect.Mouse()
		self.sx = mouseData.x - POI.UI.config:GetLeft()
		self.sy = mouseData.y - POI.UI.config:GetTop()
		self.MouseDown = true
	end, "Event.UI.Input.Mouse.Left.Down")

	border:EventAttach(Event.UI.Input.Mouse.Cursor.Move, function(self, h)
		if self.MouseDown then
			local nx, ny
			local mouseData = Inspect.Mouse()
			nx = mouseData.x - self.sx
			ny = mouseData.y - self.sy
			POI.UI.config:SetPoint("TOPLEFT", UIParent, "TOPLEFT", nx,ny)
		end
	end, "Event.UI.Input.Mouse.Cursor.Move")

	border:EventAttach(Event.UI.Input.Mouse.Left.Up, function(self, h)
		self.MouseDown = false
		MapPOI_Settings.cfgx = POI.UI.config:GetLeft()
		MapPOI_Settings.cfgy = POI.UI.config:GetTop()
	end, "Event.UI.Input.Mouse.Left.Up")
	
	POI.UI.sts_close = UI.CreateFrame("RiftButton", "POI.UI.sts_close", POI.UI.config)
	POI.UI.sts_close:SetSkin("close")
	POI.UI.sts_close:SetPoint("TOPRIGHT", POI.UI.config, "TOPRIGHT", -6,6)
	POI.UI.sts_close:EventAttach(Event.UI.Input.Mouse.Left.Click, function(self, h)
		POI.UI.config:SetVisible(false)
		POI.UI.tPOIedCz:SetKeyFocus(false)
		POI.UI.tPOIedCx:SetKeyFocus(false)
		POI.UI.tPOIedMONote:SetKeyFocus(false)
	end, "Event.UI.Input.Mouse.Left.Click")

	local trl, trr, trt, trb
	trl, trt, trr, trb = POI.UI.config:GetTrimDimensions()

	POI.UI.sb_bg = UI.CreateFrame("Frame", "POI.UI.sb_bg", POI.UI.config)
	POI.UI.sb_bg:SetWidth(4)
	POI.UI.sb_bg:SetBackgroundColor(1,0,0)
	POI.UI.sb_bg:SetLayer(10)

	POI.UI.sb = UI.CreateFrame("Frame", "POI.UI.sb", POI.UI.sb_bg)
	POI.UI.sb:SetPoint("TOPLEFT", POI.UI.sb_bg, "TOPLEFT",0,0)
	POI.UI.sb:SetHeight(20)
	POI.UI.sb:SetWidth(4)
	POI.UI.sb:SetBackgroundColor(1,1,1)
	POI.UI.sb:SetLayer(15)	
	POI.UI.sb:SetPoint("TOPLEFT", POI.UI.sb_bg, "TOPLEFT", 0,80)
	
	POI.UI.notes = {}
	
	local cxw = 0
	
	POI.UI.btn_up = UI.CreateFrame("Texture", "POI.UI.btn_up", POI.UI.sb_bg)
	POI.UI.btn_up:SetWidth(24)
	POI.UI.btn_up:SetHeight(24)
	POI.UI.btn_up:SetTexture(addon.identifier, "img/itemup.png")
	POI.UI.btn_up:SetPoint("TOPLEFT", POI.UI.config, "TOPLEFT", trl, trt)
	POI.UI.sb_bg:SetPoint("TOPLEFT", POI.UI.btn_up, "BOTTOMLEFT", 0,0)
	
	POI.UI.btn_up:EventAttach(Event.UI.Input.Mouse.Left.Click, function(self,h)
		if POI.SRCH_OS > 7 then
			POI.SRCH_OS = POI.SRCH_OS-8
		elseif POI.SRCH_OS > 0 then
			POI.SRCH_OS = 0
		end
		POI.ShowCfgPOI()
	end, "Event.UI.Input.Mouse.Left.Click")

	POI.UI.btn_dn = UI.CreateFrame("Texture", "POI.UI.btn_dn", POI.UI.sb_bg)
	POI.UI.btn_dn:SetWidth(24)
	POI.UI.btn_dn:SetHeight(24)
	POI.UI.btn_dn:SetTexture(addon.identifier, "img/itemdown.png")
	POI.UI.btn_dn:SetPoint("TOPLEFT", POI.UI.sb_bg, "BOTTOMLEFT", 0,0)

	POI.UI.btn_dn:EventAttach(Event.UI.Input.Mouse.Left.Click, function(self,h)
		if POI.SRCH_OS+8 < #MapPOI_Settings.custom[LIBZONECHANGE.currentZoneID] then
			POI.SRCH_OS = POI.SRCH_OS+8
		else
			POI.SRCH_OS = #MapPOI_Settings.custom[LIBZONECHANGE.currentZoneID] - 16
		end
		POI.ShowCfgPOI()
	end, "Event.UI.Input.Mouse.Left.Click")
	
	for x=1, 16 do
	
		local f = UI.CreateFrame("Frame", "notes.f:"..x, POI.UI.config)
		f:SetBackgroundColor(0,0,0)
		f:EventAttach(Event.UI.Input.Mouse.Cursor.In, is_mousein, "Event.UI.Input.Mouse.Cursor.In")
		f:EventAttach(Event.UI.Input.Mouse.Cursor.Out, is_mouseout, "Event.UI.Input.Mouse.Cursor.Out")
		f:EventAttach(Event.UI.Input.Mouse.Wheel.Forward, is_wheelforward, "Event.UI.Input.Mouse.Wheel.Forward")
		f:EventAttach(Event.UI.Input.Mouse.Wheel.Back, is_wheelback, "Event.UI.Input.Mouse.Wheel.Back")			
		
		f:EventAttach(Event.UI.Input.Mouse.Left.Click, function(self, h)
			local d = POI.POI_SORTED[x+POI.SRCH_OS]
			POI.UI.tPOIedCx:SetText(tostring(d.x))
			POI.UI.tPOIedCz:SetText(tostring(d.z))
			POI.UI.iPOIed:SetTexture(addon.identifier, d.itex)
			POI.EDIT_ICON_SEL = d.itex
			if d.n then
				POI.UI.tPOIedNote:SetText(d.n)
			else
				POI.UI.tPOIedNote:SetText("")
			end
		end, "Event.UI.Input.Mouse.Left.Click")

		local c = UI.CreateFrame("Text", "notes.t:"..x, f)
		c:SetFontSize(14)
		if x == 1 then
			c:SetText("123456,123456")
			POI.h14 = c:GetHeight()
			cxw = c:GetWidth()
			f:SetPoint("TOPLEFT", POI.UI.config, "TOPLEFT", trl+6, trt+24)
		else
			c:SetText(".")
			f:SetPoint("TOPLEFT", POI.UI.notes[x-1].f, "BOTTOMLEFT", 0,0)
		end
		c:SetWidth(cxw)
		f:SetHeight(POI.h14)
		c:SetPoint("TOPLEFT", f, "TOPLEFT", 0,0)
		
		f:SetWidth(POI.h14 + cxw + 256 + 4)
		
		local i = UI.CreateFrame("Texture", "notes.i:"..x, f)
		i:SetWidth(POI.h14)
		i:SetHeight(POI.h14)
		i:SetTexture(addon.identifier, "img/poi_red.png")
		i:SetPoint("TOPLEFT", c, "TOPRIGHT", 2,0)
		
		local t = UI.CreateFrame("Text", "notes.t:"..x, f)
		t:SetFontSize(14)
		t:SetWidth(256)
		t:SetPoint("TOPLEFT", i, "TOPRIGHT", 2,0)
		
		local d = UI.CreateFrame("Texture", "notes.d:"..x, f)
		d:SetWidth(POI.h14)
		d:SetHeight(POI.h14)
		d:SetTexture(addon.identifier, "img/itemdel.png")
		d:SetPoint("TOPLEFT", f, "TOPRIGHT", 2,0)
		
		d:EventAttach(Event.UI.Input.Mouse.Right.Click, function(self, h)
			table.remove(MapPOI_Settings.custom[LIBZONECHANGE.currentZoneID], POI.POI_SORTED[x+POI.SRCH_OS].k)
			POI.ShowCfgPOI()
		end, "Event.UI.Input.Mouse.Right.Click")
		
		
		POI.UI.notes[x] = {f=f, c=c, i=i, t=t, d=d}
	end
	POI.UI.sb_bg:SetHeight(16*POI.h14)
	
	POI.UI.tPOIedCoord = UI.CreateFrame("Text", "POI.UI.tPOIedCoord", POI.UI.config)
	POI.UI.tPOIedCoord:SetFontSize(12)
	POI.UI.tPOIedCoord:SetText("X, Z Coords:")
	POI.UI.tPOIedCoord:SetPoint("TOPLEFT", POI.UI.btn_dn, "BOTTOMLEFT", 0,4)
	
	POI.UI.tPOIedCx = UI.CreateFrame("RiftTextfield", "POI.UI.tPOIedCx", POI.UI.config)
	POI.UI.tPOIedCx:SetPoint("TOPLEFT", POI.UI.tPOIedCoord, "BOTTOMLEFT", 0,0)
	POI.UI.tPOIedCx:SetWidth(52)
	POI.UI.tPOIedCx:SetBackgroundColor(0.25,0,0.25,0.25)
	
	local validkeys = {
		["1"] = true,
		["2"] = true,
		["3"] = true,
		["4"] = true,
		["5"] = true,
		["6"] = true,
		["7"] = true,
		["8"] = true,
		["9"] = true,
		["0"] = true,
		["Backspace"] = true,
		["Delete"] = true,
		["Left"] = true,
		["Right"] = true,
	}
	
	POI.UI.tPOIedCx:EventAttach(Event.UI.Input.Key.Up, function(self, h, key)
		if validkeys[key] then
			local t = POI.UI.tPOIedCx:GetText()
			if string.len(t) > 5 then
				POI.UI.tPOIedCx:SetText(string.sub(t,1,5))
			end
		elseif key == "Tab" then
			POI.UI.tPOIedCz:SetKeyFocus(true)
		else
			local t = POI.UI.tPOIedCx:GetText()
			local a,b = string.gsub(t, "[^1234567890]", "")
			POI.UI.tPOIedCx:SetText(a)
		end
	end, "Event.UI.Input.Key.Up")

	POI.UI.tPOIedCz = UI.CreateFrame("RiftTextfield", "POI.UI.tPOIedCz", POI.UI.config)
	POI.UI.tPOIedCz:SetPoint("TOPLEFT", POI.UI.tPOIedCx, "TOPRIGHT", 2,0)
	POI.UI.tPOIedCz:SetWidth(52)
	POI.UI.tPOIedCz:SetBackgroundColor(0.25,0,0.25,0.25)
	
	POI.UI.tPOIedCz:EventAttach(Event.UI.Input.Key.Up, function(self, h, key)
		if validkeys[key] then
			local t = POI.UI.tPOIedCz:GetText()
			if string.len(t) > 5 then
				POI.UI.tPOIedCz:SetText(string.sub(t,1,5))
			end
		elseif key == "Tab" then
			POI.UI.tPOIedNote:SetKeyFocus(true)
		else
			local t = POI.UI.tPOIedCz:GetText()
			local a,b = string.gsub(t, "[^1234567890]", "")
			POI.UI.tPOIedCz:SetText(a)
		end
	end, "Event.UI.Input.Key.Up")

	POI.UI.iPOIed = UI.CreateFrame("Texture", "POI.UI.iPOIed", POI.UI.config)
	local h = POI.UI.tPOIedCx:GetHeight()
	POI.UI.iPOIed:SetHeight(h)
	POI.UI.iPOIed:SetWidth(h)
	POI.UI.iPOIed:SetPoint("TOPLEFT", POI.UI.tPOIedCz, "TOPRIGHT", 2,0)
	POI.UI.iPOIed:SetTexture(addon.identifier, "img/poi_red.png")
	POI.UI.iPOIed:EventAttach(Event.UI.Input.Mouse.Left.Click, function(self, h)
		POI.UI.colsframe:SetVisible(not POI.UI.colsframe:GetVisible())
	end, "Event.UI.Input.Mouse.Left.Click")
	
	local colours = { "black", "white", "red", "orange", "yellow", "green", "blue", "purple" }
	
	POI.UI.colsframe = UI.CreateFrame("Frame", "POI.UI.colsframe", POI.UI.config)
	POI.UI.colsframe:SetVisible(false)
	POI.UI.colsframe:SetLayer(100)
	POI.UI.colsframe:SetPoint("CENTER", POI.UI.iPOIed, "CENTER", 0,0)
	POI.UI.colsframe:SetWidth(64*3)
	POI.UI.colsframe:SetHeight(64*3)
	
	POI.UI.colsframe:EventAttach(Event.UI.Input.Mouse.Right.Click, function(self, h)
		POI.UI.colsframe:SetVisible(false)
	end, "Event.UI.Input.Mouse.Right.Click")
	
	POI.UI.cols = {}
	
	for x=1, 8 do
		local i = UI.CreateFrame("Texture", "cols:"..x, POI.UI.colsframe)
		i:SetWidth(64)
		i:SetHeight(64)
		i:SetLayer(150)
		i:SetTexture(addon.identifier, "img/poi_"..colours[x]..".png")
		i:EventAttach(Event.UI.Input.Mouse.Left.Click, function(self, h)
			POI.EDIT_ICON_SEL = "img/poi_"..colours[x]..".png"
			POI.UI.iPOIed:SetTexture(addon.identifier, POI.EDIT_ICON_SEL)
			POI.UI.colsframe:SetVisible(false)
		end, "Event.UI.Input.Mouse.Left.Click")
		
		if x==1 then
			i:SetPoint("TOPLEFT", POI.UI.colsframe, "TOPLEFT", 0,0)
		elseif x==2 or x==3 or x==7 or x==8 then
			i:SetPoint("TOPLEFT", POI.UI.cols[x-1].i, "TOPRIGHT", 0,0)
		elseif x==4 then
			i:SetPoint("TOPLEFT", POI.UI.cols[1].i, "BOTTOMLEFT", 0,0)
		elseif x==5 then
			i:SetPoint("TOPLEFT", POI.UI.cols[3].i, "BOTTOMLEFT", 0,0)
		elseif x==6 then
			i:SetPoint("TOPLEFT", POI.UI.cols[4].i, "BOTTOMLEFT", 0,0)
		end
			
		POI.UI.cols[x] = {i=i}
	end
	
	POI.UI.tPOIedNote = UI.CreateFrame("RiftTextfield", "POI.UI.tPOIedNote", POI.UI.config)
	POI.UI.tPOIedNote:SetPoint("TOPLEFT", POI.UI.iPOIed, "TOPRIGHT", 2,0)
	POI.UI.tPOIedNote:SetWidth(POI.h14 + cxw + 150 - h)
	POI.UI.tPOIedNote:SetBackgroundColor(0.25,0,0.25,0.25)

	POI.UI.tPOIedMONote = UI.CreateFrame("Text", "POI.UI.tPOIedMONote", POI.UI.config)
	POI.UI.tPOIedMONote:SetFontSize(12)
	POI.UI.tPOIedMONote:SetText("Mouseover note:")
	POI.UI.tPOIedMONote:SetPoint("BOTTOMLEFT", POI.UI.tPOIedNote, "TOPLEFT", 0,-4)
	
	POI.UI.iPOIadd = UI.CreateFrame("Texture", "POI.UI.iPOIed", POI.UI.config)
	POI.UI.iPOIadd:SetHeight(h)
	POI.UI.iPOIadd:SetWidth(h)
	POI.UI.iPOIadd:SetPoint("TOPLEFT", POI.UI.tPOIedNote, "TOPRIGHT", 2,0)
	POI.UI.iPOIadd:SetTexture(addon.identifier, "img/itemadd.png")
	
	POI.UI.iPOIadd:EventAttach(Event.UI.Input.Mouse.Left.Click, function(self, h)
		local x = POI.UI.tPOIedCx:GetText()
		local z = POI.UI.tPOIedCz:GetText()
		local n = POI.UI.tPOIedNote:GetText()
		local cx, cz, cn
		
		local e = false
		
		if x ~= "" and z ~= "" then
			cx = tonumber(x)
			cz = tonumber(z)
		elseif x == "" and z == "" then
			local d = Inspect.Unit.Detail("player")
			if d then
				cx = math.floor(d.coordX)
				cz = math.floor(d.coordZ)
			else
				print("Unable to find player coords!")
				e = true
			end
		else
			print("Need to specify BOTH coordinates!")
			e = true
		end
		
		if n == "" then
			cn = nil
		else
			cn = n
		end
	
		if e == false then
			POI.AddUserPOI({x=cx, z=cz, n=cn, itex=POI.EDIT_ICON_SEL})
		end
	end, "Event.UI.Input.Mouse.Left.Click")

	POI.UI.config:SetWidth(POI.UI.notes[1].f:GetWidth() + POI.h14 + 8 + trl + trr)
	POI.UI.config:SetHeight((POI.h14*16)+POI.UI.tPOIedCoord:GetHeight()+h+58+trt+trb)
end

function POI.pluginQuestObjectives()
	if POI.MAINMAPVISIBLE then
		local qdata = {}
		for k,v in pairs(Inspect.Quest.Detail(Inspect.Quest.List())) do
			if v.categoryName and v.categoryName == LIBZONECHANGE.currentZoneName then	
				if v.objective then
					for ok, ov in pairs(v.objective) do
						if ov.indicator then
							for ik, iv in pairs(ov.indicator) do
								if iv.x and iv.z then
									local n = string.format("%s\n%s", v.name, ov.description)
									local isiz =32
									if iv.radius then
										isiz = 2*LMN_Settings.scale*iv.radius
									end
									local found = false
									for qk, qv in ipairs(qdata) do
										if qv.x == iv.x and qv.z == iv.z then
											found = true
											qv.n = qv.n.."\n"..n
											break
										end
									end
									if found == false then
										table.insert(qdata, {x=iv.x, z=iv.z, isiz=isiz, isrc=addon.identifier, itex="img/poi_questMark.png", a=0.3, n=n})
									end
								end
							end
						end
					end
				end
			end
		end
		if #qdata > 0 then
			Library.LibMapNotes.AddNodes("POI: Quest",qdata)
		end
	end
end

function POI.Command_Slash_Register_Region(h, args)
	local data = {}
	
	local LM = Inspect.Addon.Detail("LibMapNotes").data	
	
	if LM.OVERLAY_X and LM.OVERLAY_Z then
	
		local pd = Inspect.Unit.Detail("player")
		
		table.insert(data, {tlx=pd.coordX-100, tlz=pd.coordZ-100, brx = pd.coordX+100, brz = pd.coordZ+100, col={r=0.5, g=0.5, b=0.5, a=1}, lab="OnPlayer"})
		table.insert(data, {tlx=pd.coordX-LM.OVERLAY_X-100, tlz=pd.coordZ-LM.OVERLAY_Z-100, brx = pd.coordX-LM.OVERLAY_X+100, brz = pd.coordZ-LM.OVERLAY_Z+100, col={r=1, g=1, b=1, a=1}, lab="Corner#1"})
		table.insert(data, {tlx=pd.coordX+LM.OVERLAY_X-100, tlz=pd.coordZ-LM.OVERLAY_Z-100, brx = pd.coordX+LM.OVERLAY_X+100, brz = pd.coordZ-LM.OVERLAY_Z+100, col={r=1, g=1, b=1}})
		table.insert(data, {tlx=pd.coordX-LM.OVERLAY_X-100, tlz=pd.coordZ+LM.OVERLAY_Z-100, brx = pd.coordX-LM.OVERLAY_X+100, brz = pd.coordZ+LM.OVERLAY_Z+100, col={r=0, g=0, b=0, a=0.5}, lab="Corner#2"})
		table.insert(data, {tlx=pd.coordX+LM.OVERLAY_X-100, tlz=pd.coordZ+LM.OVERLAY_Z-100, brx = pd.coordX+LM.OVERLAY_X+100, brz = pd.coordZ+LM.OVERLAY_Z+100, lab="Corner#4"})
		
		Library.LibMapNotes.AddRegions("Region test",data)
	else
		print("Unable to determine map coords. Please open map first, then try again!")
	end
end

POI.BuildUI()

function POI.questUpdate()
	_qstUpdate = true
end

Command.Event.Attach(Event.System.Update.Begin, POI.Event_System_Update_Begin, "Event.System.Update.Begin")
Command.Event.Attach(Library.libZoneChange.Player, POI.Event_libZoneChange_Player, "Library.LibZoneChange.Player")
Command.Event.Attach(Command.Slash.Register("poi"), POI.Command_Slash_Register_POI, "Command.Slash.Register")
Command.Event.Attach(Event.Addon.SavedVariables.Load.End, POI.Event_Addon_SavedVariables_Load_End, "Event.Addon.SavedVariables.Load.End")

Command.Event.Attach(Event.Quest.Abandon, POI.questUpdate, "Event.Quest.Abandon")
Command.Event.Attach(Event.Quest.Accept, POI.questUpdate, "Event.Quest.Accept")
Command.Event.Attach(Event.Quest.Change, POI.questUpdate, "Event.Quest.Change")
Command.Event.Attach(Event.Quest.Complete, POI.questUpdate, "Event.Quest.Complete")

print(string.format("v%s loaded.", addon.toc.Version))
