local addon, LM = ...

local default_settings = { scale = 0.75, active=true, autohide={}, pinned={}, regions={} }

LM.pCURRENTSET = nil
LM.pCURRENTREG = nil
LM.tREGISTERED_SETS = {}
LM.tREGISTERED_REGS = {}
LM.tSETLIST_IDX = {}
LM.tCALIBRATE_SET = {}
LM.tPOIFRAMES = {}
LM.tREGIONFRAMES = {}
LM.MAINMAPVISIBLE = false
LM.LMN_ACTIVE = true
LM.UPDATESETLIST = false
LM.prev_x = 0
LM.prev_z = 0

local function MergeTable(o,n)
	for k,v in pairs(n) do
		if type(v) == "table" then
			if o[k] == nil then
				o[k] = {}
			end
	 	 	if type(o[k]) == 'table' then
	 			MergeTable(o[k], n[k])
	 	 	end
		else
			if o[k] == nil then
				o[k] = v
			end
		end
	end
end

function LM.WATCHDOGWAIT(x)
	local rmx = Inspect.System.Watchdog()
	while rmx < 0.03 do
		coroutine.yield()
		rmx = Inspect.System.Watchdog()
	end
end

local math_max = math.max

local math_abs = math.abs

function LM.UpdatePositions()
	local pd = Inspect.Unit.Detail("player")
	
	if pd and pd.coordX and pd.coordZ and (math.abs(pd.coordX-LM.prev_x) > 1 or math.abs(pd.coordZ-LM.prev_z) > 1) then
		LM.prev_x = pd.coordX
		LM.prev_z = pd.coordZ
		if LM.CALIBRATE_MODE then
			LM.pCURRENTSET = LM.tCALIBRATE_SET
			LM.pCURRENTREG = {}
		else
			LM.pCURRENTSET = LM.tREGISTERED_SETS
			LM.pCURRENTREG = LM.tREGISTERED_REGS
		end
		
		local mapslope = LM.OVERLAY_Z/LM.OVERLAY_X
		
		for sk, sv in pairs(LM.pCURRENTSET) do
			if sv.CurrentStatus ~= sv.ReqdStatus then
				sv.CurrentStatus = sv.ReqdStatus
				if LM.tPOIFRAMES[sk].f[1] then
					local x = LM.tPOIFRAMES[sk].inuse
					LM.tPOIFRAMES[sk].f[x+1].f:SetVisible(true)
					LM.tPOIFRAMES[sk].f[1].f:SetVisible(false)
					LM.tPOIFRAMES[sk].inuse = 0
				end
			end

			local idx=0
			local prev_inuse = LM.tPOIFRAMES[sk].inuse
			
			if sv.CurrentStatus == LM.ST_ACTIVE then
				for k,v in pairs(sv.tData) do
					local dx = v.x - pd.coordX
					local adx = math_abs(dx)
					local dz = v.z - pd.coordZ
					local adz = math_abs(dz)
					if (adx <= LM.OVERLAY_X and adz <= LM.OVERLAY_Z) or (LMN_Settings.pinned[LIBZONECHANGE.currentZoneID][sk] == false and #sv.tData <= 32) then
					
						if adx > LM.OVERLAY_X or adz > LM.OVERLAY_Z then
							local nx, nz
							local slope = math_abs(dz/dx)
														
							if slope >= mapslope then
								-- TOP or BOTTOM
								if v.z > pd.coordZ then
									-- BOTTOM
									nz = LM.OVERLAY_Z
									if v.x > pd.coordX then
										nx = nz/slope
									else
										nx = -nz/slope
									end
								else
									-- TOP
									nz = -LM.OVERLAY_Z
									if v.x > pd.coordX then
										nx = -nz/slope
									else
										nx = nz/slope
									end
								end
							else
								-- LEFT or RIGHT
								if v.x > pd.coordX then
									-- RIGHT
									nx = LM.OVERLAY_X
									if v.z > pd.coordZ then
										nz = nx * slope
									else
										nz = -nx * slope
									end
								else
									-- LEFT
									nx = -LM.OVERLAY_X
									if v.z > pd.coordZ then
										nz = -nx * slope
									else
										nz = nx * slope
									end
								end
							end

							dx = nx
							dz = nz
						end
					
						idx = idx+1
						if idx >= #LM.tPOIFRAMES[sk].f then
							LM.MakePOIFrames(sk)
						end
						
						local fd = LM.tPOIFRAMES[sk].f[idx]
						
						if fd.fht ~= v.isiz then
							fd.fht = v.isiz
							fd.f:SetHeight(fd.fht)
							fd.f:SetWidth(fd.fht)
						end
						if fd.ftx ~= v.itex or fd.fts ~= v.isrc then
							fd.ftx = v.itex
							fd.fts = v.isrc
							fd.f:SetTexture(fd.fts, fd.ftx)
						end
						if fd.ttx ~= v.n then
							fd.ttx = v.n
							if v.n then
								fd.t:SetText(v.n)
								--fd.t:SetVisible(true)
							else
								fd.t:SetVisible(false)
							end
						end
						fd.f:SetPoint("CENTER", LM.UI.overlay, "CENTER", (LMN_Settings.scale*dx),(LMN_Settings.scale*dz))
						
						if v.a and idx==1 then
							--print("idx="..idx..",a="..v.a)
							fd.f:SetAlpha(v.a)
						else
							fd.f:SetAlpha(1)
						end
						
					end
				end
			else
				--
			end
			
			if prev_inuse ~= idx then
				for x=idx+1, prev_inuse do
					LM.tPOIFRAMES[sk].f[idx+1].t:SetVisible(false)
					LM.tPOIFRAMES[sk].f[idx+1].ttx=""
				end
				LM.tPOIFRAMES[sk].inuse = idx
				LM.tPOIFRAMES[sk].f[prev_inuse+1].f:SetVisible(true)
				LM.tPOIFRAMES[sk].f[idx+1].f:SetVisible(false)
			end
		end
		
		local regionidx = 0
		local ml = pd.coordX - LM.OVERLAY_X
		local mt = pd.coordZ - LM.OVERLAY_Z
		local mr = pd.coordX + LM.OVERLAY_X
		local mb = pd.coordZ + LM.OVERLAY_Z
		
		for sk, sv in pairs(LM.pCURRENTREG) do
			if sv.CurrentStatus ~= sv.ReqdStatus then
				sv.CurrentStatus = sv.ReqdStatus
			end

			if sv.CurrentStatus == LM.ST_ACTIVE then
				for rk,rv in pairs(sv.rData) do
					local cl = rv.tlx
					local ct = rv.tlz
					local cr = rv.brx
					local cb = rv.brz
					
					if cr < ml or cb < mt or cl > mr or ct > mb then
						--print(string.format("REG [%s]: tl=%d,%d, br=%d,%d - OUTSIDE MAP", tostring(rv.lab), rv.tlx, rv.tlz, rv.brx, rv.brz))
					else
						if cl < ml then cl = ml end
						if ct < mt then ct = mt end
						if cr > mr then cr = mr end
						if cb > mb then cb = mb end
						regionidx = regionidx+1
						if regionidx > #LM.tREGIONFRAMES then
							LM.MakeREGFrames()
						end
						LM.tREGIONFRAMES[regionidx].f:SetVisible(true)
						LM.tREGIONFRAMES[regionidx].f:SetPoint("TOPLEFT", LM.UI.overlay, "CENTER", (LMN_Settings.scale*(cl-pd.coordX)),(LMN_Settings.scale*(ct-pd.coordZ)))
						LM.tREGIONFRAMES[regionidx].f:SetPoint("BOTTOMRIGHT", LM.UI.overlay, "CENTER", (LMN_Settings.scale*(cr-pd.coordX)),(LMN_Settings.scale*(cb-pd.coordZ)))
						LM.tREGIONFRAMES[regionidx].f:SetBackgroundColor(rv.col.r, rv.col.g, rv.col.b, 0.5*rv.col.a)
						if rv.lab then
							LM.tREGIONFRAMES[regionidx].t:SetText(rv.lab)
							LM.tREGIONFRAMES[regionidx].t:SetVisible(true)
						else
							LM.tREGIONFRAMES[regionidx].t:SetVisible(false)
						end
					end
				end
			end
		end
		
		for x=regionidx+1, #LM.tREGIONFRAMES do
			LM.tREGIONFRAMES[x].f:SetVisible(false)
		end
		
	end
end

function LM.coroEvent_System_Update_Begin()
	while true do
		if LM.UPDATESETLIST then
			LM.UpdateSetList()
			LM.UPDATESETLIST = false
		end
		if LM.MAINMAPVISIBLE and LM.LMN_ACTIVE then
			LM.UpdatePositions()
		end
		coroutine.yield()
	end
end

LM.coUpdateHandler = coroutine.create(LM.coroEvent_System_Update_Begin)

function LM.Event_System_Update_Begin(h)
	if coroDead then return end
	local ok, errormsg = coroutine.resume(LM.coUpdateHandler)
	if not ok then
		coroDead = true
		error("Error in coroutine: "..errormsg)
	end
end

function LM.AddSet(n)
	if LM.tREGISTERED_SETS[n] == nil then
		LM.tREGISTERED_SETS[n] = {
			CurrentStatus = LM.ST_INACTIVE,
			ReqdStatus = LM.ST_INACTIVE,
			tData = {},
			nSetListIdx = 0
		}
		LM.UPDATESETLIST = true
	end
	if LM.tPOIFRAMES[n] == nil then
		LM.tPOIFRAMES[n] = {
			inuse = 0,
			f = {}
		}
		LM.MakePOIFrames(n)
		LM.tPOIFRAMES[n].f[1].f:SetVisible(false)
	else
		local x = LM.tPOIFRAMES[n].inuse
		LM.tPOIFRAMES[n].f[x+1].f:SetVisible(true)
		LM.tPOIFRAMES[n].f[1].f:SetVisible(false)
		LM.tPOIFRAMES[n].inuse = 0
	end
	LM.prev_x = 0
	LM.prev_z = 0	
end

function LM.Event_libZoneChange_Player(h, zn, zid)
	for k,v in pairs(LM.tPOIFRAMES) do
		if v.inuse > 0 then
			LM.tPOIFRAMES[k].f[v.inuse+1].f:SetVisible(true)
			LM.tPOIFRAMES[k].f[1].f:SetVisible(false)			
		end
	end
	if LMN_Settings.autohide[LIBZONECHANGE.currentZoneID] == nil then
		LMN_Settings.autohide[LIBZONECHANGE.currentZoneID] = {}
	end
	if LMN_Settings.pinned[LIBZONECHANGE.currentZoneID] == nil then
		LMN_Settings.pinned[LIBZONECHANGE.currentZoneID] = {}
	end
	if LMN_Settings.regions[LIBZONECHANGE.currentZoneID] == nil then
		LMN_Settings.regions[LIBZONECHANGE.currentZoneID] = {}
	end
	LM.tREGISTERED_SETS = {}
	LM.tREGISTERED_REGS = {}
	LM.tSETLIST_IDX = {}
	LM.SetActiveIcon()
	LM.UPDATESETLIST = true
end

function LM.Event_Addon_SavedVariables_Load_End(h,a)
	if a == addon.identifier then
		if LMN_Settings == nil then LMN_Settings = {} end
		MergeTable(LMN_Settings, default_settings)
		LMNACTIVE = LMN_Settings.active
	end
end

LM.BuildUI()

Command.Event.Attach(Event.System.Update.Begin, LM.Event_System_Update_Begin, "Event.System.Update.Begin")
Command.Event.Attach(Event.Addon.SavedVariables.Load.End, LM.Event_Addon_SavedVariables_Load_End, "Event.Addon.SavedVariables.Load.End")
Command.Event.Attach(Library.libZoneChange.Player, LM.Event_libZoneChange_Player, "Library.LibZoneChange.Player", Library.libZoneChange.CONST_CONSUMER)

print(string.format("v%s loaded.", addon.toc.Version))
