local addon, LM = ...

if not Library then Library = {} end
if not Library.LibMapNotes then Library.LibMapNotes = {} end

local math_floor = math.floor
local PHICON = "Data/\\UI\\texture\\global\\placeholder_icon.dds"

LM.ST_ACTIVE = 1
LM.ST_INACTIVE = 2

function LM.Validate(n,t)
	local x = false
	local z = false
	local isiz = false
	local isrc, itex
	
	if t.x and tonumber(t.x) then x = math_floor(t.x) end
	if t.z and tonumber(t.z) then z = math_floor(t.z) end
	if t.isiz and tonumber(t.isiz) then isiz = math_floor(t.isiz) end
	
	if t.isrc and t.itex == nil then
		isrc = "Rift"
		itex = PHICON
	elseif t.isrc and t.itex then
		isrc = t.isrc
		itex = t.itex
	elseif t.isrc == nil and t.itex then
		isrc = "Rift"
		itex = t.itex
	else
		isrc = addon.identifier
		itex = "img/calibrate_POI.png"
	end
	
	if x and z and isiz then
		table.insert(LM.tREGISTERED_SETS[n].tData, {x=x, z=z, isiz=isiz, isrc=isrc, itex=itex, n=t.n, a=t.a})
	end
	
end

function Library.LibMapNotes.AddRegions(n,t)
	if type(t) == "table" then
		if LM.tREGISTERED_REGS[n] == nil then
			LM.tREGISTERED_REGS[n] = {
				CurrentStatus = LM.ST_INACTIVE,
				ReqdStatus = LM.ST_INACTIVE,
				nSetListIdx = 0			
			}
			LM.UPDATESETLIST = true
		end
		
		LM.tREGISTERED_REGS[n].rData = {}
		
		if #t > 0 then
		
			for k,v in pairs(t) do
				local tlx, tlz, brx, brz
				local lab
				local col = {r=1, g=0.5, b=0, a=1}

				tlx = tonumber(v.tlx)
				tlz = tonumber(v.tlz)
				brx = tonumber(v.brx)
				brz = tonumber(v.brz)
				if tlx and tlz and brx and brz then
					local cr, cg, cb, ca
					if v.col then
						cr = tonumber(v.col.r)
						cg = tonumber(v.col.g)
						cb = tonumber(v.col.b)
						if v.col.a then
							ca = tonumber(v.col.a)
						else
							ca = 1
						end
						if cr and cg and cb and ca then
							if cr>1 or cg>1 or cb>1 or ca>1 then
								cr = cr/255
								cg = cg/255
								cb = cb/255
								ca = ca/255
							end
							col = {r=cr, g=cg, b=cb, a=ca}
						end
					end
					lab = v.lab
					table.insert(LM.tREGISTERED_REGS[n].rData, {tlx=math.floor(tlx), tlz=math.floor(tlz), brx=math.floor(brx), brz=math.floor(brz), col=col, lab=lab})
				end
			end
			LM.prev_x = 0
			LM.prev_z = 0	
		else
			LM.tREGISTERED_REGS[n].rData = {}
			LM.tREGISTERED_REGS[n].CurrentStatus = LM.ST_INACTIVE
			LM.tREGISTERED_REGS[n].ReqdStatus = LM.ST_ACTIVE		
		end
	end
end

function Library.LibMapNotes.AddNodeToSet(n,t)
	if type(t) == "table" then
		LM.AddSet(n)
		LM.Validate(n,t)
	end
end
		
function Library.LibMapNotes.AddNodes(n,t)
	if type(t) == "table" then
		if LM.tREGISTERED_SETS[n] then
			LM.tREGISTERED_SETS[n].tData = {}
		end
		LM.AddSet(n)
		if #t > 0 then
			for k,v in pairs(t) do
				LM.Validate(n,v)
			end
		else
			LM.tREGISTERED_SETS[n].tData = {}
			LM.tREGISTERED_SETS[n].CurrentStatus = LM.ST_INACTIVE
			LM.tREGISTERED_SETS[n].ReqdStatus = LM.ST_ACTIVE
		end
	end
end
