local addon, LM = ...

if not LM.UI then LM.UI = { CHECKBOX={}, POIFRAMES={} } end

LM.CALIBRATE_MODE = false

function LM.UpdateSetList()
	local num_set = 0
	local num_reg = 0
	local sorttab = {}
	LM.tSETLIST_IDX = {}
	for k,v in pairs(LM.tREGISTERED_SETS) do
		table.insert(sorttab, k)
	end
	num_set = #sorttab
	table.sort(sorttab)
	for k,v in ipairs(sorttab) do		-- k = index, v = setname
		LM.UI.CHECKBOX[k].c.TYPE = "SET"
		LM.UI.CHECKBOX[k].t:SetText(v)
		LM.UI.CHECKBOX[k].c:SetVisible(true)
		if LMN_Settings.autohide[LIBZONECHANGE.currentZoneID][v] then
			LM.tREGISTERED_SETS[v].ReqdStatus = LM.ST_INACTIVE
		else
			LM.tREGISTERED_SETS[v].ReqdStatus = LM.ST_ACTIVE
		end
		table.insert(LM.tSETLIST_IDX, v)
		LM.tREGISTERED_SETS[v].nSetListIdx = k
		LM.UI.CHECKBOX[k].c:SetChecked(LM.tREGISTERED_SETS[v].ReqdStatus == LM.ST_ACTIVE)
		LM.UI.CHECKBOX[k].c:SetPinned(LMN_Settings.pinned[LIBZONECHANGE.currentZoneID][v] == false)
	end
	
	sorttab = {}
	for k,v in pairs(LM.tREGISTERED_REGS) do
		table.insert(sorttab, k)
	end
	num_reg	= #sorttab
	table.sort(sorttab)
	for k,v in ipairs(sorttab) do
		local idx = k+num_set
		LM.UI.CHECKBOX[idx].c.TYPE = "REG"
		LM.UI.CHECKBOX[idx].t:SetText(v)
		LM.UI.CHECKBOX[idx].c:SetVisible(true)
		if LMN_Settings.regions[LIBZONECHANGE.currentZoneID][v] then
			LM.tREGISTERED_REGS[v].ReqdStatus = LM.ST_INACTIVE
		else
			LM.tREGISTERED_REGS[v].ReqdStatus = LM.ST_ACTIVE
		end
		table.insert(LM.tSETLIST_IDX, v)
		LM.tREGISTERED_REGS[v].nSetListIdx = idx
		LM.UI.CHECKBOX[idx].c:SetChecked(LM.tREGISTERED_REGS[v].ReqdStatus == LM.ST_ACTIVE)
		LM.UI.CHECKBOX[idx].c:SetPinned(false)
		
	end
	
	for x=num_set+num_reg+1, 50 do
		LM.UI.CHECKBOX[x].c:SetVisible(false)
	end
end

function LM.MakeREGFrames()
	LM.WATCHDOGWAIT()
	local oset = #LM.tREGIONFRAMES
	for x=1+oset, 10+oset do
		local f = UI.CreateFrame("Frame", "LM.tREGIONFRAMES.f:"..x, LM.UI.overlay)
		local t = UI.CreateFrame("Text", "LM.tREGIONFRAMES.t:"..x, f)
		t:SetPoint("CENTER", f, "CENTER", 0,0)
		t:SetFontSize(12)
		t:SetEffectGlow({strength=10, blurX=3, blurY=3})
		LM.tREGIONFRAMES[x] = {f=f, t=t}
	end
end

function LM.MakePOIFrames(n)
	LM.WATCHDOGWAIT()
	local st = Inspect.Time.Real()
	local oset = #LM.tPOIFRAMES[n].f
	local ctx
	for x=1+oset, 20+oset do
		if x == 1 and n == "CALIBRATION" then
			ctx = LM.UI.ctxcalibrate
		elseif x == 1 then
			ctx = LM.UI.overlay
		else
			ctx = LM.tPOIFRAMES[n].f[x-1].f
		end
		local f=UI.CreateFrame("Texture", "LM.tPOIFRAMES."..n..".f:"..x, ctx)
		f:SetVisible(true)
		f:SetWidth(32)
		f:SetHeight(32)
		--f:SetLayer(1010)
		f:SetTexture(addon.identifier, "img/calibrate_POI.png")

		local t=UI.CreateFrame("Text", "LM.tPOIFRAMES."..n..".t:"..x, LM.UI.overlay)
		t:SetFontSize(12)
		t:SetVisible(false)
		t:SetEffectGlow({strength=10, blurX=3, blurY=3})
		t:SetPoint("CENTERLEFT", f, "CENTERRIGHT", 0,0)

		f:EventAttach(Event.UI.Input.Mouse.Cursor.In, function(self, h)
			if LM.tPOIFRAMES[n].f[x].ttx then
				t:SetVisible(true)
			end
		end, "Event.UI.Input.Mouse.Cursor.In")
		f:EventAttach(Event.UI.Input.Mouse.Cursor.Out, function(self, h)
			t:SetVisible(false)
		end, "Event.UI.Input.Mouse.Cursor.Out")

		LM.tPOIFRAMES[n].f[x] = { f=f, t=t, fht=32, ftx="img/calibrate_POI.png", fts=addon.identifier, ttx=n, v=false }
	end
end

function LM.SetActiveIcon()
	if LMN_Settings.active == false then
		LM.UI.lmn_active:SetTexture(addon.identifier, "img/Red.png")
		LM.LMN_ACTIVE = false
	elseif LIBZONECHANGE.currentzoneCTY or LIBZONECHANGE.currentzoneGRP > 0 or LIBZONECHANGE.currentzonePVP then
		LM.UI.lmn_active:SetTexture(addon.identifier, "img/Purple.png")
		LM.LMN_ACTIVE = false
	else
		LM.UI.lmn_active:SetTexture(addon.identifier, "img/Green.png")
		LM.LMN_ACTIVE = true
	end
	LM.UI.calibrate:SetVisible(LM.LMN_ACTIVE)
	LM.UI.overlay:SetVisible(LM.LMN_ACTIVE)
	--LM.UI.overlay:SetVisible(LM.MAINMAPVISIBLE and LM.LMN_ACTIVE)
end

function LM.MakeCHECKBOX()
	LM.UI.frCHECKBOX = UI.CreateFrame("RiftWindow", "LM.UI.frCHECKBOX", LM.UI.mapdressing)
	LM.UI.frCHECKBOX:SetPoint("TOPRIGHT", UI.Native.MapMain, "TOPLEFT", -5,0)
	LM.UI.frCHECKBOX:SetPoint("BOTTOMRIGHT", UI.Native.MapMain, "BOTTOMLEFT", -5,0)
	LM.UI.frCHECKBOX:SetWidth(192)
	LM.UI.frCHECKBOX:SetTitle("Active Sets")
	LM.UI.frCHECKBOX.offset = 0
	local layer = LM.UI.frCHECKBOX:GetLayer() +10
	for x=1,50 do
		--local c = UI.CreateFrame("RiftCheckbox", "LM.UI.CHECKBOX.c:"..x, LM.UI.frCHECKBOX)
		local c = UI.CreateFrame("Texture", "LM.UI.CHECKBOX.c:"..x, LM.UI.frCHECKBOX)
		c.status_checked = false
		c.status_pinned = false
		function c:GetChecked(h)
			return self.status_checked
		end
		function c:GetPinned(h)
			return self.status_pinned
		end
		function c:SetPinned(status)
			if status ~= self.status_pinned then
				self.status_pinned = status
				local sn = LM.tSETLIST_IDX[x]
				if sn then
					LMN_Settings.pinned[LIBZONECHANGE.currentZoneID][sn] = not status
					LM.prev_x = 0
					LM.prev_z = 0							
				end
			end
			if self.status_checked then
				if self.status_pinned then
					c:SetTexture(addon.identifier, "img/tick_blue.png")
				else
					c:SetTexture(addon.identifier, "img/tick_green.png")
				end
			else
				c:SetTexture(addon.identifier, "img/tick_empty.png")
			end
		end
		function c:SetChecked(status)
			if status ~= self.status_checked then
				self.status_checked = status
				local sn = LM.tSETLIST_IDX[x]
				if sn then
					if self.TYPE == "SET" then
						if status then
							LM.tREGISTERED_SETS[sn].ReqdStatus = LM.ST_ACTIVE
						else
							LM.tREGISTERED_SETS[sn].ReqdStatus = LM.ST_INACTIVE
						end
						LMN_Settings.autohide[LIBZONECHANGE.currentZoneID][sn] = not status
					elseif self.TYPE == "REG" then
						if status then
							LM.tREGISTERED_REGS[sn].ReqdStatus = LM.ST_ACTIVE
						else
							LM.tREGISTERED_REGS[sn].ReqdStatus = LM.ST_INACTIVE
						end
						LMN_Settings.regions[LIBZONECHANGE.currentZoneID][sn] = not status
					end
					LM.prev_x = 0
					LM.prev_z = 0							
				end
			end
			if self.status_checked then
				if self.status_pinned then
					c:SetTexture(addon.identifier, "img/tick_blue.png")
				else
					c:SetTexture(addon.identifier, "img/tick_green.png")
				end
			else
				c:SetTexture(addon.identifier, "img/tick_empty.png")
			end
		end
		c:EventAttach(Event.UI.Input.Mouse.Left.Click, function(self, h)
			self:SetChecked(not self.status_checked)
		end, "Event.UI.Input.Mouse.Left.Click")
		c:EventAttach(Event.UI.Input.Mouse.Right.Click, function(self, h)
			self:SetPinned(not self.status_pinned)
		end, "Event.UI.Input.Mouse.Right.Click")

		c:SetWidth(24)
		c:SetHeight(24)
		c:SetLayer(layer)
		c:SetVisible(true)
		c:SetChecked(true)
		local t = UI.CreateFrame("Text", "LM.UI.CHECKBOX.t:"..x, c)
		t:SetFontSize(14)
		t:SetText("SETNUMBER:"..x)
		t:SetPoint("CENTERLEFT", c, "CENTERRIGHT", 2,0)
		t:SetLayer(layer)
		
		if x == 1 then
			local trl, trt, trr, trb
			trl, trt, trr, trb = LM.UI.frCHECKBOX:GetTrimDimensions()
			c:SetPoint("TOPLEFT", LM.UI.frCHECKBOX, "TOPLEFT", trl, trt)
		else
			c:SetPoint("TOPLEFT", LM.UI.CHECKBOX[x-1].c, "BOTTOMLEFT", 0,0)
		end
		LM.UI.CHECKBOX[x] = {c=c, t=t}
	end
end

function LM.CalculateOverlay()
	LM.OVERLAY_X = (LM.UI.overlay:GetWidth()/LMN_Settings.scale)/2
	LM.OVERLAY_Z = (LM.UI.overlay:GetHeight()/LMN_Settings.scale)/2
end

function LM.Calibrate(bMode)
	LM.CALIBRATE_MODE = not bMode
	if LM.tPOIFRAMES.CALIBRATION == nil then
		LM.tPOIFRAMES.CALIBRATION = {inuse=0, f={}}
	end
	if LM.tPOIFRAMES.CALIBRATION and LM.tPOIFRAMES.CALIBRATION.inuse > 0 then
		LM.tPOIFRAMES.CALIBRATION.f[LM.tPOIFRAMES.CALIBRATION.inuse+1].f:SetVisible(true)
	end
	LM.tPOIFRAMES.CALIBRATION.inuse = 0
	if LM.CALIBRATE_MODE then
		LM.tCALIBRATE_SET = { CALIBRATION = {CurrentStatus = LM.ST_INACTIVE, ReqdStatus = LM.ST_ACTIVE, inuse=0, tData = {} }}
		for k,v in pairs(Inspect.Map.Detail(Inspect.Map.List())) do
			if v.title then
				if not string.find(v.title, "Invasion") then
					table.insert(LM.tCALIBRATE_SET.CALIBRATION.tData, {x=v.coordX, z=v.coordZ, isiz=32, isrc = addon.identifier, itex = "img/calibrate_POI.png"})
				end
			end
		end
		if #LM.tCALIBRATE_SET.CALIBRATION.tData == 0 then
			LM.CALIBRATE_MODE = false
		end
	end
	LM.UI.calibratescale[3]:SetVisible(LM.CALIBRATE_MODE)
	LM.UI.calibratescale[2]:SetVisible(LM.CALIBRATE_MODE)
	LM.UI.calibratescale[1]:SetVisible(LM.CALIBRATE_MODE)
	
	if LM.CALIBRATE_MODE then
		LM.UI.ctxcalibrate:SetVisible(true)
		LM.UI.overlay:SetVisible(false)
	else
		LM.UI.ctxcalibrate:SetVisible(false)
		LM.UI.overlay:SetVisible(true)
	end
	LM.prev_x = 0
	LM.prev_z = 0
end

function LM.AdjustScale(s)
	if LMN_Settings.scale+s > 0.01 then
		LMN_Settings.scale = LMN_Settings.scale + s
		LM.CalculateOverlay()
	end
	LM.prev_x = 0
	LM.prev_z = 0
end

function LM.BuildUI()
	LM.UI.ctx = UI.CreateContext(addon.identifier)
	LM.UI.ctxcalibrate = UI.CreateContext(addon.identifier)
	LM.UI.mapdressing = UI.CreateContext(addon.identifier)
	LM.UI.mapdressing:SetVisible(false)
	
	local strata = "tutorial" -- UI.Native.MapMain:GetStrata()
	
	LM.UI.ctx:SetStrata(strata)
	--LM.UI.ctx:SetLayer(1000)
	LM.UI.ctxcalibrate:SetStrata(strata)
	--LM.UI.ctxcalibrate:SetLayer(1000)
	LM.UI.mapdressing:SetStrata(strata)
	--LM.UI.mapdressing:SetLayer(1000)
	
	LM.UI.overlaymask = UI.CreateFrame("Mask", "LM.UI.overlaymask", LM.UI.ctx)
	
	LM.UI.overlay = UI.CreateFrame("Frame", "LM.UI.overlay", LM.UI.overlaymask)
	--LM.UI.overlay = UI.CreateFrame("Frame", "LM.UI.overlay", LM.UI.ctx)
	LM.UI.overlay:SetVisible(false)
	LM.UI.overlay:SetMouseMasking("limited")
	--LM.UI.overlay:SetBackgroundColor(1,0,0,0.25)

	LM.UI.lmn_active = UI.CreateFrame("Texture", "LM.UI.lmn_active", LM.UI.mapdressing)
	LM.UI.lmn_active:SetVisible(true)
	LM.UI.lmn_active:SetHeight(48)
	LM.UI.lmn_active:SetWidth(48)
	LM.UI.lmn_active:SetPoint("TOPLEFT", UI.Native.MapMain, "TOPLEFT", 6,6)
	
	LM.UI.lmn_active:EventAttach(Event.UI.Input.Mouse.Left.Click, function(self, h)
		LMN_Settings.active = not LMN_Settings.active
		LM.SetActiveIcon()
	end, "Event.UI.Input.Mouse.Left.Click")

	local hf = UI.CreateFrame("RiftWindow", "hiddenframe", LM.UI.ctx)
	hf:SetVisible(false)
	hf:SetHeight(1024)
	hf:SetWidth(1024)
	local trl, trt, trr, trb
	trl, trt, trr, trb = hf:GetTrimDimensions()

	trl=trl*2
	trr=trr*2
	trb=trb*4

	LM.UI.overlaymask:SetPoint("TOPLEFT", UI.Native.MapMain, "TOPLEFT", trl, trt)
	LM.UI.overlaymask:SetPoint("BOTTOMRIGHT", UI.Native.MapMain, "BOTTOMRIGHT", -trr, -trb)

	LM.UI.overlay:SetPoint("TOPLEFT", UI.Native.MapMain, "TOPLEFT", trl, trt)
	LM.UI.overlay:SetPoint("BOTTOMRIGHT", UI.Native.MapMain, "BOTTOMRIGHT", -trr, -trb)
	
	LM.UI.overlay:EventAttach(Event.UI.Layout.Size, function()
		LM.CalculateOverlay()
	end, "Event.UI.Layout.Size")
	
	LM.MakeCHECKBOX()
	
	LM.UI.calibrate = UI.CreateFrame("RiftButton", "LM.UI.calibrate", LM.UI.mapdressing)
	LM.UI.calibrate:SetPoint("TOPRIGHT", LM.UI.overlay, "BOTTOMRIGHT", 0,2)
	LM.UI.calibrate:SetText("CALIBRATE")
	LM.UI.calibrate:EventAttach(Event.UI.Input.Mouse.Left.Click, function(self, h)
		LM.Calibrate(LM.CALIBRATE_MODE)
	end, "Event.UI.Input.Mouse.Left.Click")
	
	LM.UI.calibratescale = {}
	
	local scaledata = {
		{ n="img/scale1g.png", i="img/scale1.png", s=0.1 },
		{ n="img/scale2g.png", i="img/scale2.png", s=0.01 },
		{ n="img/scale3g.png", i="img/scale3.png", s=0.001 },
	}
	
	for x=1, 3 do
		local f = UI.CreateFrame("Texture", "LM.UI.calibratescale:"..x, LM.UI.mapdressing)
		f:SetVisible(false)
		f:SetHeight(96)
		f:SetWidth(96)
		f:SetTexture(addon.identifier, scaledata[x].n)
		f:EventAttach(Event.UI.Input.Mouse.Left.Click, function(self, h)
			LM.AdjustScale(scaledata[x].s)
		end, "Event.UI.Input.Mouse.Left.Click")
		f:EventAttach(Event.UI.Input.Mouse.Right.Click, function(self, h)
			LM.AdjustScale(-scaledata[x].s)
		end, "Event.UI.Input.Mouse.Right.Click")
		f:EventAttach(Event.UI.Input.Mouse.Wheel.Forward, function(self, h)
			LM.AdjustScale(scaledata[x].s)
		end, "Event.UI.Input.Mouse.Wheel.Forward")
		f:EventAttach(Event.UI.Input.Mouse.Wheel.Back, function(self, h)
			LM.AdjustScale(-scaledata[x].s)
		end, "Event.UI.Input.Mouse.Wheel.Back")
		f:EventAttach(Event.UI.Input.Mouse.Cursor.In, function(self, h)
			self:SetTexture(addon.identifier, scaledata[x].i)
		end, "Event.UI.Input.Mouse.Cursor.In")
		f:EventAttach(Event.UI.Input.Mouse.Cursor.Out, function(self, h)
			self:SetTexture(addon.identifier, scaledata[x].n)
		end, "Event.UI.Input.Mouse.Cursor.Out")
		
		LM.UI.calibratescale[x] = f
	end
	
	LM.UI.calibratescale[3]:SetPoint("TOPRIGHT", UI.Native.MapMain, "BOTTOMRIGHT", -10,4)
	LM.UI.calibratescale[2]:SetPoint("TOPRIGHT", LM.UI.calibratescale[3], "TOPLEFT", -10,0)
	LM.UI.calibratescale[1]:SetPoint("TOPRIGHT", LM.UI.calibratescale[2], "TOPLEFT", -10,0)

	UI.Native.MapMain:EventAttach(Event.UI.Native.Loaded, function()
		LM.MAINMAPVISIBLE = not LM.MAINMAPVISIBLE
		LM.UI.mapdressing:SetVisible(LM.MAINMAPVISIBLE)
		if LM.MAINMAPVISIBLE then
			LM.CalculateOverlay()
			LM.Calibrate(true)
			LM.UI.overlay:SetVisible(LM.LMN_ACTIVE)
		else
			LM.UI.overlay:SetVisible(false)
		end
	end, "Event.UI.Native.Loaded")	
end
